package com.demo;

import java.awt.Font;
import java.awt.Color;
import java.awt.BasicStroke;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.plot.Marker;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.time.Hour;
import org.jfree.data.time.Minute;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.TextAnchor;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;
import org.jfree.chart.annotations.XYAnnotation;
import org.jfree.chart.annotations.XYPointerAnnotation;
import org.jfree.chart.annotations.XYDrawableAnnotation;

public class TimeSeriesMarker extends ApplicationFrame
{
    private static final long serialVersionUID = 1L;
    
	public TimeSeriesMarker(final String title) {

        super(title);
        final XYDataset  data       = Dataset.createSuppliersBids();
        final JFreeChart chart      = createChart(data);
        final ChartPanel chartPanel = new ChartPanel(chart);
        
        chartPanel.setPreferredSize(new java.awt.Dimension(560, 480));

        setContentPane(chartPanel);
    }
    private JFreeChart createChart(final XYDataset data)
    {
        final JFreeChart chart = ChartFactory.createScatterPlot(
            "Пример аннотаций и маркеров", null, null, data, 
            PlotOrientation.VERTICAL, true, true, false);
        // Настройка диаграммы ...
        final XYPlot plot = chart.getXYPlot();

        // Определение временной оси
        final DateAxis domainAxis = new DateAxis(null);
        domainAxis.setUpperMargin(0.50);
        plot.setDomainAxis(domainAxis);

        // Скрытие осевых линий
        plot.setAxisOffset(new RectangleInsets (1.0, 1.0, 1.0, 1.0));
        ValueAxis axis = plot.getDomainAxis();
        axis.setAxisLineVisible (false);
        axis = plot.getRangeAxis();
        axis.setAxisLineVisible (false);      

        final ValueAxis rangeAxis = plot.getRangeAxis();
        rangeAxis.setUpperMargin(0.30);
        rangeAxis.setLowerMargin(0.50);
        
        Marker start = createRangeMarker(200.0, "Стартовая цена предложений",
                                  RectangleAnchor.BOTTOM_RIGHT,
                                  TextAnchor.TOP_RIGHT, Color.red);
        Marker target = createRangeMarker(175.0, "Выгодная цена",
                                   RectangleAnchor.TOP_RIGHT,
                                   TextAnchor.BOTTOM_RIGHT, Color.blue);
        plot.addRangeMarker(start );
        plot.addRangeMarker(target);

        final Hour hour = Dataset.getHour(2);
        double millis = hour.getFirstMillisecond();
        
        Marker originalEnd = createRangeMarker(millis, "Закрытие торговой сессии (02:00)",
                                        RectangleAnchor.TOP_LEFT,
                                        TextAnchor.TOP_RIGHT, Color.red);
        plot.addDomainMarker(originalEnd);

        final Minute min = new Minute(15, hour);
        millis = min.getFirstMillisecond();
        Marker currentEnd = createRangeMarker(millis, "Завершение сделок (02:15)",
                                              RectangleAnchor.TOP_RIGHT,
                                              TextAnchor.TOP_LEFT, Color.blue);
        plot.addDomainMarker(currentEnd);

     // Аннотация лучшего предложения
        final Hour h = Dataset.getHour(2);
        final Minute m = new Minute(10, h);
        millis = m.getFirstMillisecond();
        final CircleDrawer cd = new CircleDrawer(Color.red, new BasicStroke(1.0f), null);
        final XYAnnotation bestBid = new XYDrawableAnnotation(millis, 163.0, 11, 11, cd);
        plot.addAnnotation(bestBid);
        
        plot.addAnnotation(createPtrAnnotation(millis));

        return chart;
    }
    private Marker createRangeMarker(final double value, final String caption, 
    		                         final RectangleAnchor ranchor,
    		                         final TextAnchor tanchor,
    		                         final Color color)
    {
    	final Marker marker = new ValueMarker(value);
    	marker.setPaint(color);
    	marker.setLabel(caption);
    	marker.setLabelAnchor(ranchor);
    	marker.setLabelTextAnchor(tanchor);

    	return marker;
    }
    private XYPointerAnnotation createPtrAnnotation(final double millis)
    {
        XYPointerAnnotation pointer;
        pointer = new XYPointerAnnotation("Выгодное предложение", millis, 163.0,
                                                              3.0 * Math.PI / 4.0);
        pointer.setBaseRadius(35.0);
        pointer.setTipRadius(10.0);
        pointer.setFont(new Font("SansSerif", Font.PLAIN, 9));
        pointer.setPaint(Color.blue);
        pointer.setTextAnchor(TextAnchor.HALF_ASCENT_RIGHT);
        return pointer;
    }
    
    public static void main(final String[] args)
    {
        final TimeSeriesMarker demo = new TimeSeriesMarker("Пример маркеров");
        demo.pack();
        RefineryUtilities.centerFrameOnScreen(demo);
        demo.setVisible(true);
    }
}
