package com.demo;

import java.awt.Color;
import java.text.SimpleDateFormat;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.general.SeriesException;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.RefineryUtilities;

public class TimeSeriesRandow extends ApplicationFrame
{
	private static final long serialVersionUID = 1L;
	private static final String TITLE = "Пример TimeSeries";
	
	public TimeSeriesRandow(final String title)
	{
        super(title);
        
        XYDataset  dataset    = createDataset();
        JFreeChart chart      = createChart(dataset);
        ChartPanel chartPanel = new ChartPanel(chart);
        
        chartPanel.setPreferredSize(new java.awt.Dimension(560, 360));
        chartPanel.setMouseZoomable(true, true);
        setContentPane(chartPanel);
    }
	 private XYDataset createDataset() 
	 {
		 TimeSeries series = new TimeSeries("Random Data");
		 Day current = new Day(1, 1, 2011);
		 double value = 100.0;
		 for (int i = 0; i < 2200; i++) {
			 try {
				 value = value + Math.random() - 0.5;
				 series.add(current, new Double(value));
				 current = (Day) current.next();
			 }
			 catch (SeriesException e) {}
		 }
		 return new TimeSeriesCollection(series);
	 }
	 private JFreeChart createChart(final XYDataset dataset) 
	 {
		 JFreeChart chart = ChartFactory.createTimeSeriesChart(
				 TITLE, null, null, dataset, false, false, false );

		 XYPlot plot = chart.getXYPlot();
		 // Определение отступа меток делений
		 plot.setAxisOffset(new RectangleInsets (1.0, 1.0, 1.0, 1.0));
		 // Скрытие осевых линий
		 ValueAxis axis = plot.getDomainAxis();
		 axis.setAxisLineVisible (false);
		 axis = plot.getRangeAxis();
		 axis.setAxisLineVisible (false);
	     // Фон и цвет сетки графика 
		 plot.setBackgroundPaint(new Color(232, 232, 232));
		 plot.setDomainGridlinePaint(Color.lightGray);
		 plot.setRangeGridlinePaint (Color.lightGray);
		 
		 // Определение временной оси
		 DateAxis daxis = (DateAxis) plot.getDomainAxis();
		 // Формат отображения осевых меток
		 daxis.setDateFormatOverride(new SimpleDateFormat("MM.yyyy"));
		 return chart;
	 }
	 public static void main(final String[] args)
	 {
		 final TimeSeriesRandow demo = new TimeSeriesRandow(TITLE);
		 demo.pack();
		 RefineryUtilities.positionFrameRandomly(demo);
		 demo.setVisible(true);
	 }
}
